local AssignSoldier = ...

AssignSoldier:registerEnumValue ("BUILDING_PART_TYPE", "EXTENSION")

-- Function list
AssignSoldier:registerAsset({
        DataType = "BUILDING_FUNCTION_TRAINING_GROUND",
        Id = _assignSoldierPrefix .. "_FUNCTION",
        Name = _assignSoldierPrefix .. "_FUNCTION_NAME",
        WorkerCapacity = 1,
        RelatedJob = { Job = "SOLDIER", Behavior = "BEHAVIOR_SOLDIER" },
        TrainingBehaviorList = { "BEHAVIOR_SOLDIER_TRAINING" }, --_assignSoldierPrefix .. "_BEHAVIOR_TREE" },
        VillagerProfile = "PROFILE_SOLDIER"
})

-- Function list
AssignSoldier:registerAsset({
		DataType = "BUILDING_FUNCTION_ASSIGNABLE",
                Id = _assignSoldierPrefix .. "_ASSIGNABLE_FUNCTION",
                AuthorizedFunctionList = _assignSoldierPrefix .. "_FUNCTION_LIST"
})

AssignSoldier:registerAsset({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = _assignSoldierPrefix .. "_FUNCTION_LIST",
                AssetFunctionList = {
                        _assignSoldierPrefix .. "_FUNCTION"
                }
})

if _flagSecurity2 then
    AssignSoldier:registerAsset({
        DataType = "BUILDING_FUNCTION_WORKPLACE",
        Id = _assignSoldierPrefix .. "_BUILDING_FUNCTION_SECURITY_GUARD2",
	  Name = "BUILDING_FUNCTION_SECURITY_GUARD_NAME",
        WorkerCapacity = 1,
        RelatedJob = { 
            Job = "JOB_SECURITY_GUARD", 
            Behavior = _assignSoldierPrefix .. "_BEHAVIOR_TREE"
	  },
	  InputInventoryCapacity = {
	  },
        ResourceListNeeded = {
	  },
	  ResourceProduced = {
		{ Resource = "RESOURCE_SECURITY", Quantity = 0 },
	  }
    })
    AssignSoldier:overrideAsset({
		DataType = "BUILDING_FUNCTION_LIST",
                Id = _assignSoldierPrefix .. "_FUNCTION_LIST",
                AssetFunctionList = {
                        Action = "APPEND",
                       _assignSoldierPrefix .. "_BUILDING_FUNCTION_SECURITY_GUARD2",
                       "BUILDING_FUNCTION_SECURITY_CONSTABLE",
                       "BUILDING_FUNCTION_SECURITY_PRISON",
                }
    })
end

-- Apply building asset processor
AssignSoldier:registerAssetProcessor("models/assignSoldier/keepTrap.fbx", {
    DataType = "BUILDING_ASSET_PROCESSOR"
})
AssignSoldier:registerAssetProcessor("models/assignSoldier/smallGate.fbx", {
    DataType = "BUILDING_ASSET_PROCESSOR"
})
AssignSoldier:registerAssetProcessor("models/assignSoldier/octogonalTower.fbx", {
    DataType = "BUILDING_ASSET_PROCESSOR"
})

AssignSoldier:registerAssetId("models/assignSoldier/octogonalTower.fbx/Materials/redRoof", _assignSoldierPrefix .. "_OTOWER_RED_TILE_ROOF_MATERIAL")
AssignSoldier:registerAssetId("models/assignSoldier/octogonalTower.fbx/Materials/murChateau", _assignSoldierPrefix .. "_OTOWER_STONE_MANOR_WALL_MATERIAL")

function registerAEnd(_nameFbxFile, _nameFbx, _nameAsset, _workplaceCustom, _isVisibleWhenBuilt)
    -- Register Main prefab nodes
    AssignSoldier:registerAssetId("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameFbx .. "Part", _assignSoldierPrefix .. "_" .. _nameAsset .. "_PREFAB")

    -- Register the building part
    AssignSoldier:registerAsset({
	DataType = "BUILDING_PART",
	Id = _assignSoldierPrefix .. "_" .. _nameAsset .. "_PART",
	Name = _assignSoldierPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = _assignSoldierPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = StoneGateSetPrefix .. "_ENTRANCE_PARTSETLIST_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = _assignSoldierPrefix .. "_" .. _nameAsset .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 0 }
		}}
	},
        IsVisibleWhenBuilt = _isVisibleWhenBuilt,
--        AssetBuildingFunction = _assignSoldierPrefix .. "_ASSIGNABLE_FUNCTION"
    })
    AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameFbx .. "Part", {
        DataType = "ASSIGN_SOLDIER_PATH_AEND_COMP",
        Enabled = true,
        WorkplaceCustom = _workplaceCustom
    })
    if _flagSecurity2 then
        AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameFbx .. "Part", {
            DataType = "COMP_SECURITY_POINTS",
            BasePoints = 4,
            WorkerPoints = 2
        })
        AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameFbx .. "Part", {
            -- DataType = "COMP_SECURITY_OVERLAY"
            DataType = "COMP_SECURITY_OVERLAY_INIT"
        })
    end
end

registerAEnd("keepTrap", "access/keepTrapAccess", "KEEP_TRAP_ACCESS", "KeepTrapAccess", false)
registerAEnd("smallGate", "access/smallGateAccessR", "SMALL_GATE_ACCESS_R", "KeepTrapAccess", false)
registerAEnd("smallGate", "access/smallGateAccessL", "SMALL_GATE_ACCESS_L", "KeepTrapAccess", false)
registerAEnd("octogonalTower", "access/octogonalAccess", "OCTOGONAL_TOWER_ACCESS", "OctogonalAccess", false)
    AssignSoldier:registerPrefabComponent("models/assignSoldier/keepTrap.fbx/Prefab/access/keepTrapAccessPart", {
        DataType = "COMP_BUILDING_PART",
        BuildingPartType = _optionAccessNodeType
    })
    for i=0, 3 do
        AssignSoldier:registerPrefabComponent("models/assignSoldier/keepTrap.fbx/Prefab/access/keepTrapAccessPart/Attach_StoneGateAccess_askta" .. i, {
	    DataType = "COMP_BUILDING_ATTACH_NODE",
	    AttachNodeTypeList = { _optionAccessNodeType },
            AllowedAngles = {-45, 45}
        })
    end
    AssignSoldier:registerPrefabComponent("models/assignSoldier/smallGate.fbx/Prefab/access/smallGateAccessRPart", {
        DataType = "COMP_BUILDING_PART",
        BuildingPartType = _optionAccessNodeType
    })
    for i=0, 1 do
        AssignSoldier:registerPrefabComponent("models/assignSoldier/smallGate.fbx/Prefab/access/smallGateAccessRPart/Attach_StoneGateAccess_rampart_low_assgar" .. i, {
	    DataType = "COMP_BUILDING_ATTACH_NODE",
	    AttachNodeTypeList = { _optionAccessNodeType },
            AllowedAngles = {-45, 45}
        })
    end
    AssignSoldier:registerPrefabComponent("models/assignSoldier/smallGate.fbx/Prefab/access/smallGateAccessLPart", {
        DataType = "COMP_BUILDING_PART",
        BuildingPartType = _optionAccessNodeType
    })
    for i=0, 1 do
        AssignSoldier:registerPrefabComponent("models/assignSoldier/smallGate.fbx/Prefab/access/smallGateAccessLPart/Attach_StoneGateAccess_rampart_low_assgal" .. i, {
	    DataType = "COMP_BUILDING_ATTACH_NODE",
	    AttachNodeTypeList = { _optionAccessNodeType },
            AllowedAngles = {-45, 45}
        })
    end
    AssignSoldier:registerPrefabComponent("models/assignSoldier/octogonalTower.fbx/Prefab/access/octogonalAccessPart", {
        DataType = "COMP_BUILDING_PART",
        BuildingPartType = _optionAccessNodeType
    })
    for i=0, 7 do
        AssignSoldier:registerPrefabComponent("models/assignSoldier/octogonalTower.fbx/Prefab/access/octogonalAccessPart/Attach_StoneGateAccess_asoa" .. i, {
	    DataType = "COMP_BUILDING_ATTACH_NODE",
	    AttachNodeTypeList = { _optionAccessNodeType },
            AllowedAngles = {-45, 45}
        })
    end

function registerKeepTrapPlatform(_nameFbxFile, _nameFbx)
    AssignSoldier:configurePrefabFlagList("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/access/" .. _nameFbx .. "Part", { "PLATFORM" })

    AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/access/" .. _nameFbx .. "Part/KEEP_TRAP_ACCESS_GUARDING_LOCATION", {
            DataType = "ASSIGN_SOLDIER_ACCESS_GUARDING_LOCATION_COMP",
            Enabled = true
    })
    AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/access/" .. _nameFbx .. "Part/KEEP_TRAP_ACCESS_INTERMEDIATE_LOCATION_UP", {
            DataType = "ASSIGN_SOLDIER_ACCESS_INTERMEDIATE_LOCATION_UP_COMP",
            Enabled = true
    })
    AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/access/" .. _nameFbx .. "Part/KEEP_TRAP_ACCESS_INTERMEDIATE_LOCATION_DOWN", {
            DataType = "ASSIGN_SOLDIER_ACCESS_INTERMEDIATE_LOCATION_DOWN_COMP",
            Enabled = true
    })
    AssignSoldier:registerPrefabComponent("models/assignSoldier/" .. _nameFbxFile .. ".fbx/Prefab/access/" .. _nameFbx .. "Part/KEEP_TRAP_ACCESS_INTERMEDIATE_LOCATION_OUT", {
            DataType = "ASSIGN_SOLDIER_ACCESS_INTERMEDIATE_LOCATION_OUT_COMP",
            Enabled = true
    })
end

registerKeepTrapPlatform("keepTrap", "keepTrapAccess")
registerKeepTrapPlatform("smallGate", "smallGateAccessR")
registerKeepTrapPlatform("smallGate", "smallGateAccessL")
registerKeepTrapPlatform("octogonalTower", "octogonalAccess")

